@@ -1,6 +1,7 @@
 from __future__ import unicode_literals, absolute_import
 
 import io
+import os
 import re
 import abc
 import csv
@@ -18,8 +19,11 @@ from ._compat import (
     suppress,
     map,
     FileNotFoundError,
+    IsADirectoryError,
     NotADirectoryError,
+    PermissionError,
     pathlib,
+    PYPY_OPEN_BUG,
     ModuleNotFoundError,
     MetaPathFinder,
     email_message_from_string,
@@ -103,6 +107,8 @@ class EntryPoint(collections.namedtuple('EntryPointBase', 'name value group')):
     @classmethod
     def _from_text(cls, text):
         config = ConfigParser()
+        # case sensitive: https://stackoverflow.com/q/1611799/812183
+        config.optionxform = str
         try:
             config.read_string(text)
         except AttributeError:  # pragma: nocover
@@ -362,8 +368,9 @@ class MetadataPathFinder(NullFinder, DistributionFinder):
 
     @staticmethod
     def _switch_path(path):
-        with suppress(Exception):
-            return zipp.Path(path)
+        if not PYPY_OPEN_BUG or os.path.isfile(path):  # pragma: no branch
+            with suppress(Exception):
+                return zipp.Path(path)
         return pathlib.Path(path)
 
     @classmethod
@@ -386,7 +393,8 @@ class PathDistribution(Distribution):
         self._path = path
 
     def read_text(self, filename):
-        with suppress(FileNotFoundError, NotADirectoryError, KeyError):
+        with suppress(FileNotFoundError, IsADirectoryError, KeyError,
+                      NotADirectoryError, PermissionError):
             return self._path.joinpath(filename).read_text(encoding='utf-8')
     read_text.__doc__ = Distribution.read_text.__doc__
 
